import os
import sys
import json
from pwd import getpwnam
from subprocess import check_output

from landscape.configuration import (
    setup, register, LandscapeSetupConfiguration, ConfigurationError,
    stop_client_and_disable_init_script)
from landscape.deployment import Configuration
from landscape.sysvconfig import SysVConfig


_config = Configuration()


def log(message):
    """Log a message via Juju."""
    return check_output(["juju-log", message])


def get_client_config():
    """
    Return a L{Configuration} object with the current client configuration.
    """
    _config.reload()
    return _config


def _run_juju_tool(command, params=None):
    """Run specified Juju tool and parse json output."""

    cmd = [command, "--format", "json"]
    if params:
        cmd.extend(params)
    output = check_output(cmd)
    return json.loads(output) if output else {}


def get_relation_config():
    """Get relation configuration from Juju."""
    return _run_juju_tool("relation-get")


def get_service_config():
    """Get service configuration from Juju."""
    return _run_juju_tool("config-get")


def update_client_config(service_config):
    """Update the client config.

    @param service_config: A dict containing the config items to be
        updated. Dashes in the key values will be converted to
        underscore, to match the Landscape configuration.

    An attempt to register the client will be made, if the client wasn't
    configured before.
    """
    client_config = get_client_config()
    for key, value in service_config.items():
        if key != "origin":
            setattr(client_config, key.replace("-", "_"), value)
        if key == "registration-key":
            # Use registration_password additionally for
            # backwards compatibility with landscape-client in precise.
            setattr(client_config, "registration_password", value)
    client_config.write()
    return try_to_register()


class ErrorHandler:

    exit_code = 0

    def flag_error(self, exit_code):
        """Disable the client and indicate an error was encountered.

        The client is disabled, so that the config-changed hook can be
        re-run after the configuration error has been resolve. If the client
        wouldn't be disabled, try_to_register() would see that the client is
        configured and wouldn't attempt to register the client again.
        """
        # Disable the client so that the config-change hook can be re-run
        # after the error has been resolved.
        stop_client_and_disable_init_script()
        self.exit_code = exit_code


def clear_registration():
    """
    Do steps necessary to clear the client registration, so that next time
    it will re-register
    """
    stop_client_and_disable_init_script()


def try_to_register():
    """Try to register the client if needed.

    If the client has already been configured, not registration attempt
    will be made. The same is true if a computer title hasn't been set,
    since that means that the juju-info-relation-joined hasn't been run,
    which means that it's expected that the configuration will be broken
    at that point.

    The following keys need to be set for an attempt to be made:
     - account_name, computer_title

    If an error is encountered, the client will be disabled and the hook
    calling this function will exit with an error code.
    """
    error_handler = ErrorHandler()

    sysvconfig = SysVConfig()
    configured = sysvconfig.is_configured_to_run()
    config = LandscapeSetupConfiguration()
    config.load([])
    config.silent = True
    if configured:
        print "Client already registered, skipping"
        return 0
    if not config.get("account_name") or not config.get("computer_title"):
        print "Need account-name and computer-title to proceed, skipping"
        return 0
    try:
        setup(config)
    except ConfigurationError as error:
        print >> sys.stderr, "Configuration error: %s" % (str(error),)
        error_handler.flag_error(1)
    else:
        register(config, on_error=error_handler.flag_error)
    return error_handler.exit_code


def chown(filename):
    """Change file owner and group to landscape."""
    user = getpwnam("landscape")
    os.chown(filename, user.pw_uid, user.pw_gid)
